@@ -1,8 +1,8 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2017, 2018, 2019, 2020, 2021, 2022, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2017, 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2020, 2021 Paul Hoogendijk <paul@dezyne.org>
-;;; Copyright © 2018, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2018, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2017, 2018 Johri van Eerd <vaneerd.johri@gmail.com>
 ;;; Copyright © 2017, 2018, 2019 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2017 Henk Katerberg <hank@mudball.nl>
@@ -29,18 +29,19 @@
 (define-module (dzn commands verify)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
+
   #:use-module (ice-9 getopt-long)
 
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
-  #:use-module (dzn goops)
+  #:use-module (dzn ast goops)
   #:use-module (dzn ast)
+  #:use-module (dzn code)
+  #:use-module (dzn code makreel)
+  #:use-module (dzn command-line)
+  #:use-module (dzn commands code)
+  #:use-module (dzn commands parse)
   #:use-module (dzn config)
   #:use-module (dzn parse)
-  #:use-module (dzn code makreel)
-  #:use-module (dzn commands parse)
-  #:use-module (dzn commands code)
   #:use-module (dzn verify pipeline)
-  #:use-module (dzn command-line)
   #:export (parse-opts
             main))
 
@@ -49,11 +50,14 @@
           '((all (single-char #\a))
             (help (single-char #\h))
             (import (single-char #\I) (value #t))
+            (jitty (single-char #\j))
             (model (single-char #\m) (value #t))
+            (no-constraint (single-char #\C))
+            (no-interfaces)
+            (no-non-compliance (single-char #\D))
+            (no-unreachable (single-char #\U))
             (out (value #t))
-            (queue-size (single-char #\q) (value #t))
-            (queue-size-defer (value #t))
-            (queue-size-external (value #t))))
+            (queue-size (single-char #\q) (value #t))))
 	 (options (getopt-long args option-spec))
 	 (help? (option-ref options 'help #f))
 	 (files (option-ref options '() '()))
@@ -69,16 +73,17 @@ Usage: dzn verify [OPTION]... DZN-FILE
 Check DZN-FILE for verification errors in Dezyne models
 
   -a, --all                   keep going after first error
+  -C, --no-constraint         do not use a constraining process
+  -D, --no-non-compliance     report deadlock upon non-compliance
   -h, --help                  display this help and exit
   -I, --import=DIR+           add DIR to import path
+  -j, --jitty                 run lps2lts with --rewriter=jittyc
   -m, --model=MODEL           restrict verification to model MODEL
+      --no-interfaces         skip interface verification
       --out=FORMAT            produce output FORMAT (use \"help\" for a list)
-  -q, --queue-size=SIZE       use queue size=SIZE for verification [~a]
-      --queue-size-defer=SIZE
-                              use defer queue size=SIZE for verification [~a]
-      --queue-size-external=SIZE
-                              use external queue size=SIZE for verification [~a]
-" (%queue-size) (%queue-size-defer) (%queue-size-external))
+  -U, --no-unreachable        skip the unreachable code check
+  -q, --queue-size=SIZE       use queue size=SIZE for verification [3]
+")
 	(exit (or (and usage? EXIT_OTHER_FAILURE) EXIT_SUCCESS))))
     options))
 
@@ -96,25 +101,33 @@ Check DZN-FILE for verification errors in Dezyne models
                                  (lambda _ (ast:get-model ast model-name))
                                  #:backtrace? debug?
                                  #:file-name file-name)))
-         (root (makreel:normalize ast))
-         (queue-size (option-ref options 'queue-size (%queue-size)))
-         (queue-size-defer (option-ref options 'queue-size-defer
-                                       (%queue-size-defer)))
-         (queue-size-external (option-ref options 'queue-size-external
-                                          (%queue-size-external))))
-    (parameterize ((%queue-size queue-size)
-                   (%queue-size-defer queue-size-defer)
-                   (%queue-size-external queue-size-external))
-      (cond (out
-             (let ((formats (verification:formats)))
-               (unless (member out formats)
-                 (format #t "formats:~a\n" (string-join (verification:formats) "\n  " 'prefix))
-                 (exit EXIT_OTHER_FAILURE))
-               (let* ((model (call-with-handle-exceptions
-                              (lambda _ (ast:get-model ast model-name))
-                              #:backtrace? debug?
-                              #:file-name file-name))
-                      (model-name (ast:dotted-name model)))
-                 (verification:partial root model-name #:out out))))
-            (else
-             (exit (verification:verify options root #:all? all? #:model-name model-name)))))))
+         (no-unreachable? (command-line:get 'no-unreachable)))
+    (parameterize ((%no-unreachable? no-unreachable?))
+      (let ((root (makreel:om ast)))
+        (when (and=> model ast:imported?)
+          (let ((name (ast:dotted-name model)))
+            (format (current-error-port)
+                    "~a:error: cannot verify imported model: ~a\n"
+                    (ast:source-file root)
+                    name)
+            (format (current-error-port)
+                    "~a:info: ~a imported from here\n"
+                    (ast:source-file model)
+                    name))
+          (exit EXIT_OTHER_FAILURE))
+        (cond
+         (out
+          (let ((formats (verification:formats)))
+            (unless (member out formats)
+              (format #t "formats:~a\n" (string-join (verification:formats)
+                                                     "\n  " 'prefix))
+              (exit EXIT_OTHER_FAILURE))
+            (let* ((model (call-with-handle-exceptions
+                           (lambda _ (ast:get-model ast model-name))
+                           #:backtrace? debug?
+                           #:file-name file-name))
+                   (model-name (ast:dotted-name model)))
+              (verification:partial root model-name #:out out))))
+         (else
+          (exit (verification:verify options root #:all? all?
+                                     #:model-name model-name))))))))
