@@ -25,7 +25,6 @@
 ;;; Code:
 
 (define-module (dzn parse)
-
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
   #:use-module (srfi srfi-71)
@@ -36,17 +35,14 @@
   #:use-module (ice-9 poe)
   #:use-module (ice-9 pretty-print)
 
+  #:use-module (dzn ast display)
+  #:use-module (dzn ast goops)
+  #:use-module (dzn ast parse)
+  #:use-module (dzn ast)
   #:use-module (dzn command-line)
-  #:use-module (dzn display)
   #:use-module (dzn parse peg)
-  #:use-module (dzn parse ast)
   #:use-module (dzn parse peg)
-  #:use-module (dzn wfc)
-
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
-  #:use-module (dzn goops)
-
-  #:declarative? #f
+  #:use-module (dzn ast wfc)
 
   #:export (file->ast
             file->stream
@@ -236,8 +232,7 @@ using CONTENT-ALIST to transform locations."
                                            #:file-name file-name
                                            #:working-directory working-directory)))))
                alist))
-         (ast (expand-imports ast-alist))
-         (ast (annotate-ast ast)))
+         (ast (expand-imports ast-alist)))
     (when (> (dzn:debugity) 1)
       (ast:pretty-print ast (current-error-port)))
     ast))
