@@ -2,7 +2,7 @@
 ;;;
 ;;; Copyright © 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2019 Rob Wieringa <rma.wieringa@gmail.com>
-;;; Copyright © 2018, 2019, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2018, 2019, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2021, 2022 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
@@ -24,32 +24,33 @@
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
   #:use-module (srfi srfi-71)
+
   #:use-module (ice-9 match)
   #:use-module (ice-9 rdelim)
   #:use-module (ice-9 readline)
   #:use-module (ice-9 regex)
   #:use-module (ice-9 string-fun)
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
-  #:use-module (dzn config)
-  #:use-module (dzn goops)
+
+  #:use-module (dzn ast goops)
+  #:use-module (dzn ast serialize)
   #:use-module (dzn ast)
   #:use-module (dzn command-line)
+  #:use-module (dzn config)
   #:use-module (dzn misc)
-  #:use-module (dzn serialize)
   #:use-module (dzn vm ast)
-  #:use-module (dzn vm goops)
   #:use-module (dzn vm evaluate)
+  #:use-module (dzn vm goops)
   #:use-module (dzn vm runtime)
   #:export (%debug
             %debug?
             %next-input
+            %queue-size
             %startup-info
             %strict?
 
             append-port-trace
             action->trigger
             assign
-            async-event?
             block
             blocked-on-action?
             blocked-on-boundary-collateral-release
@@ -64,6 +65,7 @@
             defer-labels
             dequeue
             dequeue-external
+            end-of-trail-labels
             enqueue
             enqueue-external
             external-trigger?
@@ -76,6 +78,7 @@
             graft-locals
             in-event?
             is-status?
+            label->string
             label?
             labels
             make-implicit-illegal
@@ -154,6 +157,9 @@
   (when (%debug?)
     (format (current-error-port) fmt arg ...)))
 
+;;; The size of the component queues and external queues.
+(define %queue-size (make-parameter 3))
+
 ;; Is the input trail to be matched exactly?
 (define %strict? (make-parameter #f))
 
@@ -379,7 +385,12 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
        ports)))))
 
 (define-method (labels (pc <program-counter>))
-  (append (labels) (rtc-labels pc) (return-labels pc) (defer-labels pc)))
+  (delete-duplicates
+   (append (labels)
+           (rtc-labels pc)
+           (end-of-trail-labels pc)
+           (return-labels pc)
+           (defer-labels pc))))
 
 (define-method (label? (o <string>))
   (and (member o (labels)) o))
@@ -410,6 +421,48 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
                (port-names (map (compose .name .ast) ports)))
           (map (cute format #f "~a.<rtc>" <>) port-names)))))
 
+(define (label->string o)
+  (match o
+    (($ <action>)
+     (trigger->string o))
+    (($ <enum-literal>)
+     (string-append (ast:name (.type.name o)) ":" (.field o)))
+    (($ <literal>)
+     (label->string (.value o)))
+    ((? (is? <trigger>))
+     (trigger->string o))
+    (($ <trigger-return>)
+     (let ((prefix (or (and=> (.port.name o) (cute string-append <> ".")) "")))
+       (string-append prefix (label->string (.event.name o)))))
+    ((? (is? <model>))
+     #f)
+    ((? string?)
+     o)
+    (#f
+     "false")
+    (#t
+     "true")))
+
+(define (end-of-trail-labels pc)
+  (let* ((status (.status pc))
+         (ast (and=> status .ast))
+         (instance (.instance pc))
+         (end-of-trail-labels? (and (is-a? status <end-of-trail>)
+                                    (or (is-a? ast <action>)
+                                        (is-a? ast <trigger-return>)))))
+    (if (not end-of-trail-labels?) '()
+        (let* ((ast (.ast status))
+               (port-name (if (not instance) (.port.name ast)
+                              (string-join (runtime:instance->path instance)
+                                           ".")))
+               (ast (trigger->string (clone ast #:port.name #f)))
+               (labels (ast:label* status))
+               (labels (map label->string labels))
+               (labels (if (is-a? (%sut) <runtime:port>) labels
+                           (map (cute string-append port-name "." <>)
+                                labels))))
+          labels))))
+
 (define-method (return-labels (pc <program-counter>))
   (let* ((blocked (.blocked pc))
          (released (.released pc))
@@ -444,7 +497,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
       ((path ... "true") (make <literal> #:value "true"))
       ((path ... "false") (make <literal> #:value "false")))))
 
-(define-method (string->value (type <int>) (o <string>))
+(define-method (string->value (type <subint>) (o <string>))
   (let ((value (string-split o #\.)))
     (match value
       ((path ... number) (make <literal> #:value (string->number number))))))
@@ -613,7 +666,6 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
          (instance (.instance pc))
          (pc (reset-handling! pc))
          (pc (make <program-counter>
-               #:async (.async pc)
                #:id (pc:next-id)
                #:blocked (acons port pc (.blocked pc))
                #:collateral (.collateral pc)
@@ -622,6 +674,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
                #:defer (.defer pc)
                #:external-q (.external-q pc)
                #:released (.released pc)
+               #:skip-compliance? (.skip-compliance? pc)
                #:state (.state pc)
                #:trail (.trail pc))))
     (%debug "  ~s ~s <block> ~a [~a] => [~a]\n"
@@ -763,7 +816,6 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
          (blocked-port (or (blocked-port pc instance)
                            (blocked-on-boundary? (.previous pc))))
          (pc (make <program-counter>
-               #:async (.async pc)
                #:id (pc:next-id)
                #:blocked (.blocked pc)
                #:collateral (acons blocked-port pc (.collateral pc))
@@ -851,7 +903,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
   (let* ((external-q (.external-q pc))
          (instance (.instance pc))
          (q (or (assoc-ref external-q instance) '())))
-    (if (= (length q) (%queue-size-external))
+    (if (= (length q) (%queue-size))
         (let ((error (make <queue-full-error> #:ast ast #:instance instance
                            #:message "queue-full")))
           (clone pc #:status error))
@@ -992,6 +1044,9 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
 (define-method (get-variables (pc <program-counter>))
   ((compose .variables get-state) pc))
 
+(define-method (get-variables (pc <program-counter>) (instance <runtime:instance>))
+  (.variables (get-state pc instance)))
+
 (define-method (set-variables (pc <program-counter>) (o <list>))
   (set-state pc (clone (get-state pc) #:variables o)))
 
@@ -1035,7 +1090,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
   (unless (or (is-a? o <formal>) (is-a? o <variable>))
     (error "range-error" o))
   (let ((type (.type o)))
-    (and (is-a? type <int>)
+    (and (is-a? type <subint>)
          (let ((range (.range type))
                (value (.value value)))
            (and (or (< value (.from range))
@@ -1192,35 +1247,36 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
   (let* ((instance (.instance o))
          (path (runtime:instance->path instance))
          (path (match path
-                 (("sut" path ...) path)
+                 (("sut" path ...) (if (null? path) '("sut") path))
                  (_ path)))
          (variables (map (match-lambda ((x . y)
                                         (format #f "~a=~a" x (->sexp y))))
                          (.variables o)))
-         (q (.q o)))
+         (q (.q o))
+         (q-string (string-join (map trigger->string q) ","))
+         (variables (if (null? q) variables
+                        (cons (string-append "q=" q-string) variables))))
     (and (not (equal? path '("client")))
          (or (pair? variables) (pair? q))
          (string-append
           (string-join path ".")
-          (if (pair? path) "=" "")
+          (if (pair? path) ":" "")
           "["
           (string-join variables ",\n")
-          (if (null? q) ""
-              (string-append "q=" (string-join (map trigger->string q) ",")))
           "]"))))
 
 (define-method (state->string (o <system-state>))
-  (let* ((state-list (.state-list o))
-         (state-list (if (is-a? (%sut) <runtime:port>) state-list
-                         (filter (disjoin (compose (is? <runtime:component>) .instance)
-                                          (compose ast:requires? .ast .instance))
-                                 state-list))))
+  (let ((state-list (.state-list o)))
     (string-join (filter-map state->string state-list) "\n")))
 
-(define-method (async-ports (pc <program-counter>))
-  (map (match-lambda ((timeout port . proc)
-                      (runtime:dotted-name port)))
-       (.async pc)))
+(define-method (state->string (o <system-state>) include-provides?)
+  (let* ((state-list (.state-list o))
+         (state-list (if include-provides? state-list
+                         (filter
+                          (disjoin (compose (is? <runtime:component>) .instance)
+                                   (compose ast:requires? .ast .instance))
+                          state-list))))
+    (string-join (filter-map state->string state-list) "\n")))
 
 (define-method (pc->string (o <program-counter>))
   (match (.status o)
@@ -1232,7 +1288,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
      "<deadlock>")
     (_
      (string-join
-      (cons (state->string (.state o))
+      (cons (state->string (.state o) (not (.skip-compliance? o)))
             (append
              (let* ((instance (.instance o))
                     (statement (.statement o))
@@ -1296,7 +1352,6 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
                    ,@(map trigger->string (blocked-on-boundary-statements o))))
              (if (null? (.external-q o)) '()
                  (list (external-q->string (.external-q o))))
-             (async-ports o)
              (let ((defer (.defer o)))
                (if (null? defer) '()
                    `("defer:"
@@ -1326,8 +1381,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
                      (string-join
                       (map (compose runtime:dotted-name car) (.collateral o))
                       ","))))
-             (map (compose runtime:dotted-name car) (.collateral o))
-             (async-ports o)))
+             (map (compose runtime:dotted-name car) (.collateral o))))
       "\n"))))
 
 (define-method (pc->hash (o <program-counter>))
@@ -1339,7 +1393,10 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
 ;;;
 
 (define-method (is-status? (type <class>))
-  (lambda (pc) (is-a? (.status pc) type)))
+  (lambda (pc)
+    (let ((status (.status pc)))
+      (and (is-a? status type)
+           status))))
 
 (define (provides/requires-trigger? string ast:provides/requires? ast:in/out?)
   (let* ((trigger (string->trigger string))
@@ -1433,7 +1490,6 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
        (ast:eq? (.statement a) (.statement b))
        (equal? (serialize (.state a)) (serialize (.state b)))
        (equal? (.trail a) (.trail b))
-       (equal? (async-ports a) (async-ports b))
        (ast:equal? (.blocked a) (.blocked b))
        (ast:equal? (.collateral a) (.collateral b))
        (equal? (.released a) (.released b))
@@ -1455,7 +1511,6 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
        (ast:equal? (.status a) (.status b))
        (pc:ast:equal? (.statement a) (.statement b))
        (equal? (serialize (.state a)) (serialize (.state b)))
-       (equal? (async-ports a) (async-ports b))
        (ast:equal? (.blocked a) (.blocked b))
        (ast:equal? (.collateral a) (.collateral b))
        (equal? (.released a) (.released b))
@@ -1473,12 +1528,6 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
 (define (trace-head:eq? a b)
   (pc-equal? (car a) (car b)))
 
-(define-method (async-event? (pc <program-counter>) event)
-  (and (string? event)
-       (not (member event (labels)))
-       (not (return-trigger? event))
-       (pair? (.async pc))))
-
 (define-method (defer-event? (pc <program-counter>) event)
   (and (string? event)
        (not (member event (labels)))
