@@ -36,24 +36,32 @@
   #:use-module (gnu packages maths)
   #:use-module (gnu packages pkg-config))
 
+(define %source-dir (getcwd))
+
+;; scp kluit.dezyne.org:.../download/dezyne-2.10.0.tar.gz .
+;; guix download ./dezyne-2.10.0.tar.gz
+
+;; This makes `git am 0001-guix-dezyne-Update-to-2.13.2.patch' work.
+(define guile-3.0-latest guile-next)
+
 (define-public dezyne
   (package
     (name "dezyne")
-    (version #!dezyne!# "2.16.4")
+    (version #!dezyne!# "2.17.0.rc2")
     (source
      (origin
        (method url-fetch)
        (uri (string-append "https://dezyne.org/download/dezyne/"
                            name "-" version ".tar.gz"))
        (sha256
-        (base32 #!dezyne!# "07k1ivraqzwshwjipdhd2p9ac5dvq5a9vkwzyscrbq70kzs3dk21"))))
+        (base32 #!dezyne!# "1j23xnpkngj04k15q9nwdmgag45ylbrkzc141dhrzfjggk0fis9a"))))
     (inputs (list bash-minimal
-                  guile-3.0
+                  guile-3.0-latest
                   guile-json-4
                   guile-readline
                   mcrl2-minimal
                   sed))
-    (native-inputs (list guile-3.0 pkg-config))
+    (native-inputs (list guile-3.0-latest pkg-config))
     (build-system gnu-build-system)
     (arguments
      (list
@@ -65,19 +73,21 @@
             (lambda _
               (setenv "GUILE_AUTO_COMPILE" "0")))
           (add-after 'install 'install-readmes
-            (lambda _
-              (let* ((base (string-append #$name "-" #$version))
-                     (doc (string-append #$output "/share/doc/" base)))
+            (lambda* (#:key outputs #:allow-other-keys)
+              (let* ((out (assoc-ref outputs "out"))
+                     (base (string-append #$name "-" #$version))
+                     (doc (string-append out "/share/doc/" base)))
                 (mkdir-p doc)
                 (copy-file "NEWS" (string-append doc "/NEWS")))))
           (add-after 'install 'wrap-binaries
-            (lambda _
-              (let* ((bash #$(this-package-input "bash-minimal"))
-                     (guile #$(this-package-input "guile-3.0"))
-                     (json #$(this-package-input "guile-json"))
-                     (mcrl2 #$(this-package-input "mcrl2-minimal"))
-                     (readline #$(this-package-input "guile-readline"))
-                     (sed #$(this-package-input "sed"))
+            (lambda* (#:key inputs outputs #:allow-other-keys)
+              (let* ((out (assoc-ref outputs "out"))
+                     (bash (assoc-ref %build-inputs "bash-minimal"))
+                     (guile (assoc-ref %build-inputs "guile-next"))
+                     (json (assoc-ref %build-inputs "guile-json"))
+                     (mcrl2 (assoc-ref %build-inputs "mcrl2-minimal"))
+                     (readline (assoc-ref %build-inputs "guile-readline"))
+                     (sed (assoc-ref %build-inputs "sed"))
                      (effective (read
                                  (open-pipe* OPEN_READ
                                              "guile" "-c"
@@ -88,15 +98,15 @@
                                  (string-append sed "/bin")))
                      (scm-dir (string-append "/share/guile/site/" effective))
                      (scm-path
-                      (list (string-append #$output scm-dir)
+                      (list (string-append out scm-dir)
                             (string-append json scm-dir)
                             (string-append readline scm-dir)))
                      (go-dir (string-append "/lib/guile/" effective
                                             "/site-ccache/"))
-                     (go-path (list (string-append #$output go-dir)
+                     (go-path (list (string-append out go-dir)
                                     (string-append json go-dir)
                                     (string-append readline go-dir))))
-                (wrap-program (string-append #$output "/bin/dzn")
+                (wrap-program (string-append out "/bin/dzn")
                   `("PATH" ":" prefix ,path)
                   `("GUILE_AUTO_COMPILE" ":" = ("0"))
                   `("GUILE_LOAD_PATH" ":" prefix ,scm-path)
