@@ -3,7 +3,7 @@
 // Copyright © 2015, 2016, 2017, 2018, 2019, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 // Copyright © 2016 Rob Wieringa <rma.wieringa@gmail.com>
 // Copyright © 2016 Henk Katerberg <hank@mudball.nl>
-// Copyright © 2015, 2016, 2017, 2019, 2020, 2022, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
+// Copyright © 2015, 2016, 2017, 2019, 2020, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 //
 // This file is part of dzn-runtime.
 //
@@ -25,13 +25,13 @@
 // Code:
 
 #include <dzn/locator.hh>
+#include <dzn/meta.hh>
 #include <dzn/pump.hh>
 #include <dzn/runtime.hh>
 
 #include <algorithm>
 #include <cassert>
 #include <iterator>
-#include <thread>
 
 namespace dzn
 {
@@ -51,15 +51,15 @@ namespace dzn
     auto pump = locator.try_get<dzn::pump>();
     return !pump ? 1 : pump->coroutine_id();
   }
-  void port_block(const dzn::locator& locator, void* component, void* port)
+  void port_block(const dzn::locator& locator, dzn::component* component, void* port)
   {
     locator.get<dzn::pump>().block(locator.get<dzn::runtime>(), component, port);
   }
-  void port_release(const dzn::locator& locator, void* component, void* port)
+  void port_release(const dzn::locator& locator, dzn::component* component, void* port)
   {
     locator.get<dzn::pump>().release(locator.get<dzn::runtime>(), component, port);
   }
-  void collateral_block(const dzn::locator& locator, void* component)
+  void collateral_block(const dzn::locator& locator, dzn::component* component)
   {
     locator.get<dzn::pump>().collateral_block(locator.get<dzn::runtime>(), component);
   }
@@ -107,7 +107,7 @@ namespace dzn
   , paused(false)
   , current_coroutine(0)
   , switch_context()
-  , task(dzn::thread::defer(std::ref(*this)))
+  , task(std::async(std::launch::async, std::ref(*this)))
   {}
   pump::~pump()
   {
@@ -289,7 +289,7 @@ namespace dzn
       context();
     }
   }
-  void pump::collateral_block(dzn::runtime& runtime, void* component)
+  void pump::collateral_block(dzn::runtime& runtime, dzn::component* component)
   {
     auto self = find_self(coroutines);
     debug.rdbuf() && debug << "[" << self->id << "] collateral_block"
@@ -353,7 +353,7 @@ namespace dzn
       unblocked.clear();
     }
   }
-  void pump::block(dzn::runtime& runtime, void* component, void* port)
+  void pump::block(dzn::runtime& runtime, dzn::component* component, void* port)
   {
     auto self = find_self (coroutines);
     runtime.blocked(component) = self->id;
@@ -399,7 +399,7 @@ namespace dzn
     runtime.reset_skip_block(component);
     runtime.blocked(component) = 0;
   }
-  bool pump::collateral_release_skip_block(void* component)
+  bool pump::collateral_release_skip_block(dzn::component* component)
   {
     bool have_collateral = false;
     collateral_blocked.reverse();
@@ -423,7 +423,7 @@ namespace dzn
     collateral_blocked.reverse();
     return have_collateral;
   }
-  void pump::release(dzn::runtime& runtime, void* component, void* port)
+  void pump::release(dzn::runtime& runtime, dzn::component* component, void* port)
   {
     runtime.set_skip_block(component, port);
 
